//******************************************************************************
//                                                                             *
//COPYRIGHT: Copyright (C) International Business Machines Corp., 1992,1994.   *
//                                                                             *
//DISCLAIMER OF WARRANTIES:                                                    *
//  The following [enclosed] code is sample code created by IBM                *
//  Corporation.  This sample code is not part of any standard IBM product     *
//  and is provided to you solely for the purpose of assisting you in the      *
//  development of your applications.  The code is provided "AS IS",           *
//  without warranty of any kind.  IBM shall not be liable for any damages     *
//  arising out of your use of the sample code, even if they have been         *
//  advised of the possibility of such damages.                                *
//                                                                             *
//******************************************************************************

#include "store.hpp"
#include <iomanip.h>
//******************************************************************************
// File Name: store.cpp                                                        *
//                                                                             *
// Note: This file illustrates how you can design your own object class        *
//       based on the table object class generated by the DAX Data Access      *
//       tool. This file keeps all the member functions of your own object     *
//       class. The database connection is established in the constructor      *
//       and is disconnected in the destructor.                                *
//                                                                             *
//******************************************************************************
//******************************************************************************
// Constructor: Establish the connection to the data base.                     *
//******************************************************************************
Store::Store(IString shopDBNameArg): dsm(shopDBNameArg, "USERID", "PASSWORD") {  // The database name for the shop
  try {
    // establish the connection to the datastore now.
    dsm.connect();
    // change the price list object to readonly
 
  } catch (IException &exc) {
    cout << "Failed: Connection" << endl;
    cout << "Error id: " << exc.errorId() << endl;
    cout << "Error Text: " << exc.text() << endl;
    cout << "Error Class: " << exc.name() << endl;
  } catch(...) {
    cout << "Unknown exception in connect" << endl;
  }
}

//******************************************************************************
// Destructor: Disconnect the connection.                                      *
//******************************************************************************
Store::~Store() {
  try {
    // disconnect the connection
    dsm.disconnect();
  } catch (IException &exc) {
    cout << "Failed: Disconnection" << endl;
    cout << "Error id: " << exc.errorId() << endl;
    cout << "Error Text: " << exc.text() << endl;
    cout << "Error Class: " << exc.name() << endl;
  } catch(...) {
    cout << "Unknown exception in disconnect" << endl;
  }
}

//******************************************************************************
// Function Name: generatePriceList
// Note : This member function generates a price list for the store.
//        The data printed in the list is a subset of the inventory
//        database.
//******************************************************************************
void Store::generatePriceList() {
  try {
    // call refresh to get a new copy of the price list.
    priceListMgr.refresh();

    // set up the cursor for going through each item.
    IVSequence<PriceList *>::Cursor cursor(*(priceListMgr.items()));

    // loop until the end of the list.
    cout << "Prod. ID   Description            Price " << endl;
    cout << "========   =====================  ===== " << endl;
    for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext()) {
      priceListPtr = priceListMgr.items()->elementAt(cursor);
      cout << priceListPtr->prodnumber() << ' ';
      cout << priceListPtr->proddescription() << "   $";
 
      cout << setiosflags(ios::fixed) << setprecision(2) << priceListPtr->listingprice() << endl;
    }
  } catch (IDSAccessError &exc) {
    cout << "Failed: generatePriceList " << endl;
    cout << "Error Text:" << exc.text() << endl;
    cout << "Error id:"  << exc.errorId() << endl;
    cout << "Error Class: " << exc.name() << endl;
    if (exc.errorId() == DAX_REF_SQLERR) {
      cout << "SQLCODE: " << exc.getSqlca().sqlcode << endl;
    } 
  } catch(...) {
    cout << "Unknown exception in generatePriceList" << endl;
  }
}

//******************************************************************************
// Function Name: postingInventory                                             *
// Note: This member function updates the inventory table with the received    *
//       received order table.                                                 *
//******************************************************************************
void Store::postingInventory() {
  double newAveCost;                      // new average cost after update
  double newListingPrice;                 // new listing price after update
  long   newQuantity;                     // new quantity after update
  char   selectConditionArg[] = "postedflag = 0";  // selection condition

  try {
    // get a copy of all the rows that meet the select condition.
    receivedOrderMgr.select(selectConditionArg);

    // set up the cursor to go through each item.
    IVSequence<Receivedorder *>::Cursor cursor(*(receivedOrderMgr.items()));

    // loop until the end of the list.
    for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext()) {

      // get the pointer to the current received order row.
      receivedOrderPtr = receivedOrderMgr.items()->elementAt(cursor);

      // Use the product number as the key to locate the row of the item
      // in the inventory table.
      inventoryObj.setProdnumber(receivedOrderPtr->prodnumber());
      inventoryObj.retrieve();

      // calculate the new quantity and the new average cost
      newQuantity = inventoryObj.onhandquantity() +
                    receivedOrderPtr->receivedquantity();
      newAveCost  = (inventoryObj.averagecost() * inventoryObj.onhandquantity() +
                    receivedOrderPtr->totalcost()) / newQuantity;
      newListingPrice = newAveCost * 1.50;     // 50% mark up of the new cost

      // update the row with the new data
      inventoryObj.setOnhandquantity(newQuantity);
      inventoryObj.setAveragecost(newAveCost);
      inventoryObj.setListingprice(newListingPrice);
      inventoryObj.update();

      // reset the posted flag so that it will not be used again.
      receivedOrderPtr->setPostedflag(POSTED);
      receivedOrderPtr->update();
    }
  } catch (IDSAccessError &exc) {
    cout << "Failed: postingInventory " << endl;
    cout << "Error id:"  << exc.errorId() << endl;
    cout << "Error Text:" << exc.text() << endl;
    cout << "Error Class: " << exc.name() << endl;
    if ((exc.errorId() == DAX_SEL_SQLERR) || 
        (exc.errorId() == DAX_RET_SQLERR) ||
        (exc.errorId() == DAX_UPD_SQLERR)) {
      cout << "SQLCODE: " << exc.getSqlca().sqlcode << endl;
    }
  } catch (IException &exc) {
    cout << "Failed: postingInventory" << endl;
    cout << "Error id: " << exc.errorId() << endl;
    cout << "Error Text: " << exc.text() << endl;
    cout << "Error Class: " << exc.name() << endl;
  } catch(...) {
    cout << "Unknown exception in generatePriceList" << endl;
  }
}

//******************************************************************************
// Function: addReceivedOrder                                                  *
// Note: Whenever an order is received, a new entry is added to the received   *
//       order table                                                           *
//******************************************************************************
void Store::addReceivedOrder(
       short   receivedNumberArg,     //received order number
       IString prodNumberArg,         // product number
       long    receivedQuantityArg,   // received order quantity
       double  totalCostArg,          // total cost of this order
       IString receivedDateArg) {     // date of the received order
  try {
    receivedOrderObj.setPostedflag      (NOT_POSTED);
    receivedOrderObj.setProdnumber      (prodNumberArg);
    receivedOrderObj.setReceivednumber  (receivedNumberArg);
    receivedOrderObj.setReceiveddate    (receivedDateArg);
    receivedOrderObj.setReceivedquantity(receivedQuantityArg);
    receivedOrderObj.setTotalcost       (totalCostArg);
    receivedOrderObj.add();
  } catch (IDSAccessError &exc) {
    cout << "Failed: addReceivedOrder " << endl;
    cout << "Error id:"  << exc.errorId() << endl;
    cout << "Error Text:" << exc.text() << endl;
    cout << "Error Class: " << exc.name() << endl;
    if (exc.errorId() == DAX_ADD_SQLERR) {
      cout << "SQLCODE: " << exc.getSqlca().sqlcode << endl;
    }
  } catch(...) {
    cout << "Unknown exception in addReceiveOrder" << endl;
  }
}

//******************************************************************************
// Function: transaction
// Note: This function commits or rollbacks the activities between transactions
//******************************************************************************
void Store::transaction(commitflag flagArg) {  // commit or rollback
  try {
    if (flagArg == COMMIT) {
      dsm.commit();
    } else {
      dsm.rollback();
    }
  } catch (IException &exc) {
    cout << "Failed: transaction " << endl;
    cout << "Error id:"  << exc.errorId() << endl;
    cout << "Error Text:" << exc.text() << endl;
    cout << "Error Class: " << exc.name() << endl;
  } catch(...) {
    cout << "Unknown exception in transaction" << endl;
  }
}

//******************************************************************************
// Function: addNewItem
// Note: This function add a new row to the inventory table
//******************************************************************************
void Store::addNewItem(
       IString prodNumberArg,       // product number
       IString prodDescriptionArg,  // product description
       long    onHandQuantityArg,   // on hand quantity
       double  averageCostArg,      // average cost per unit
       double  listingPriceArg) {   // listing price per unit
  try {
    inventoryObj.setProdnumber      (prodNumberArg);
    inventoryObj.setProddescription (prodDescriptionArg);
    inventoryObj.setOnhandquantity  (onHandQuantityArg);
    inventoryObj.setAveragecost     (averageCostArg);
    inventoryObj.setListingprice    (listingPriceArg);
    inventoryObj.add();
  } catch (IDSAccessError &exc) {
    cout << "Failed: addNewItem " << endl;
    cout << "Error id:"  << exc.errorId() << endl;
    cout << "Error Text:" << exc.text() << endl;
    cout << "Error Class: " << exc.name() << endl;
  } catch(...) {
    cout << "Unknown exception in addNewItem" << endl;
  }
}

//******************************************************************************
// Function: displayItem
// Note: This function displays a new row in the inventory table
//******************************************************************************
void Store::displayItem() {
      cout << inventoryObj.prodnumber() << ' ';
      cout << inventoryObj.proddescription() << ' ';
      cout << inventoryObj.listingprice() << endl;
}


